function [PCPAG] = incorporatePK_PAG(PAG, PK)

global nnodes;
global found;
nnodes = length(PAG.G);
found = false(nnodes,nnodes,2);
updateFound(PAG.G);

sat = search_all(PAG.G,PAG,PK);
if(~sat)
    PCPAG = {};
    return;
end

pag = PAG.G;
for i = 1:nnodes
    for j = 1:nnodes
        if(found(i,j,1) && ~found(i,j,2))
            pag(i,j) = 2;
        elseif(~found(i,j,1) && found(i,j,2))
            pag(i,j) = 3;
        end
    end
end

PCPAG.G = pag;
PCPAG.PK = {};
closure = transitiveClosurePag_mex(pag);

for i = 1:length(PK)
    if(PK{i}.path && ~closure(PK{i}.s,PK{i}.t))
        PCPAG.PK{length(PCPAG.PK)+1} = PK{i};
    end
end

end

function [x y] = getNextEdge(pag)
global found;

[x y] = find(pag == 1 & ~found(:,:,1) & ~found(:,:,2),1);

if(~isempty(x))
    return;
end

[x y] = find(pag == 1 & (~found(:,:,1) | ~found(:,:,2)),1);

if(~isempty(x))
    return;
end

x = [];
y = [];
end

function updateFound(pag)
global found;
global nnodes;

for i = 1:nnodes
    for j = 1:nnodes
        if(pag(i,j) == 2)
            found(i,j,1) = true;
        elseif(pag(i,j) == 3)
            found(i,j,2) = true;
        end
    end
end

end

function [bool] = allKnown(pag)
global found;

f1 = ~found(:,:,1);
f2 = ~found(:,:,2);

if(any(any((pag == 1 & (f1 | f2)) | (pag == 2 & f1) | (pag == 3 & f2))))
    bool = false;
else
    bool = true;
end

end

function [sat] = search_all(pag, PAG, PK)
[X,Y] = getNextEdge(pag);

if(isempty(X))
    if(~allKnown(pag))
        if(any(any(pag == 1)))
            [sat pag] = hardrule(pag,PAG,PK);
            
            if(sat == true)
                updateFound(pag);
            end
        else
            closure = transitiveClosurePag_mex(pag);
            sat = ~hasCycle(closure) && ~hasAlmostCycle(pag,closure) && ~PKViolated(pag,PK,closure) && ~dncsViolated(pag, PAG.dnc) && ~independenciesViolated(PAG.sepSet, pag, closure);
            if(sat == true)
                updateFound(pag);
            end
        end
    end
    return;
end

% Test arrow
pag1 = pag;
pag1(X,Y) = 2;
pag1 = FCI_rules_apply(pag1, 0);
closure = transitiveClosurePag_mex(pag1);
sat1 = ~allKnown(pag1) && ~hasCycle(closure) && ~hasAlmostCycle(pag1,closure) && ~PKViolated(pag1,PK,closure) && search_all(pag1,PAG, PK);

% Test tail
if(pag(Y,X) == 3)
    sat2 = false;
else
    pag2 = pag;
    pag2(X,Y) = 3;
    pag2(Y,X) = 2;
    
    pag2 = FCI_rules_apply(pag2, 0);
    closure = transitiveClosurePag_mex(pag2);
    sat2 = ~allKnown(pag2) && ~hasCycle(closure) && ~hasAlmostCycle(pag2,closure) && ~PKViolated(pag2,PK,closure) && search_all(pag2,PAG, PK);
end

sat = sat1 || sat2;

end

function [sat pag] = hardrule(pag,PAG, PK)
[X,Y] = find(pag == 1,1);

if(isempty(X))
    closure = transitiveClosurePag_mex(pag);
    sat = ~hasCycle(closure) && ~hasAlmostCycle(pag,closure) && ~PKViolated(pag,PK,closure) && ~dncsViolated(pag, PAG.dnc) && ~independenciesViolated(PAG.sepSet, pag, closure);
    return;
end

% Test arrow
pag1 = pag;
pag1(X,Y) = 2;
pag1 = FCI_rules_apply(pag1, 0);
closure = transitiveClosurePag_mex(pag1);

if(~allKnown(pag1) && ~hasCycle(closure) && ~hasAlmostCycle(pag1,closure) && ~PKViolated(pag1,PK,closure))
    [sat pag1] = hardrule(pag1,PAG,PK);
    if(sat)
        pag = pag1;
        return;
    end
end

% Test tail
if(pag(Y,X) ~= 3)
    pag2 = pag;
    pag2(X,Y) = 3;
    pag2(Y,X) = 2;
    pag2 = FCI_rules_apply(pag2, 0);
    closure = transitiveClosurePag_mex(pag2);
    
    if(~allKnown(pag2) && ~hasCycle(closure) && ~hasAlmostCycle(pag2,closure) && ~PKViolated(pag2,PK,closure))
        [sat pag2] = hardrule(pag2,PAG,PK);
        if(sat)
            pag = pag2;
            return;
        end
    end
end

sat = false;

end
