function [PCPDAG] = incorporatePK_PDAG(PDAG, PK)

global nnodes;
global found;
nnodes = length(PDAG.G);
found = false(nnodes,nnodes);

sat = search_all(PDAG.G,PK);
if(~sat)
    PCPDAG = {};
    return;
end

pdag = PDAG.G;
for i = 1:nnodes
    for j = i+1:nnodes
        if(found(i,j) && ~found(j,i))
            pdag(i,j) = 2;
            pdag(j,i) = 3;
        end
        
        if(~found(i,j) && found(j,i))
            pdag(i,j) = 3;
            pdag(j,i) = 2;
        end
    end
end

PCPDAG.G = pdag;
PCPDAG.PK = {};
closure = transitiveClosurePag_mex(pdag);

for i = 1:length(PK)
    if(PK{i}.path && ~closure(PK{i}.s,PK{i}.t))
        PCPDAG.PK{length(PCPDAG.PK)+1} = PK{i};
    end
end

end

function [bool] = allKnown(pdag)
global found;

f1 = ~found;
f2 = f1';

if(any(any((pdag == 1 & (f1 | f2)) | (pdag == 2 & f1) | (pdag == 3 & f2))))
    bool = false;
else
    bool = true;
end

end


function [sat] = search_all(pdag, PK)
global found;

[X,Y] = find(pdag == 1 & ~(found & found'),1);

if(isempty(X))
    if(~allKnown(pdag))
        if(any(any(pdag == 1)))
            [sat pdag] = hardrule(pdag,PK);
            if(sat == true)
                found = found | (pdag == 2 & pdag' == 3);
            end
        else
            sat = true;
            found = found | (pdag == 2 & pdag' == 3);
        end
    end
    return;
end

% Test arrow
pdag1 = pdag;
pdag1(X,Y) = 2;
pdag1(Y,X) = 3;
pdag1 = PC_rules_apply(pdag1, 0);
closure1 = transitiveClosurePag_mex(pdag1);

sat1 = ~allKnown(pdag1) && ~hasCycle(closure1) && ~PKViolated(pdag1,PK,closure1) && search_all(pdag1,PK);

% Test tail
pdag2 = pdag;
pdag2(X,Y) = 3;
pdag2(Y,X) = 2;
pdag2 = PC_rules_apply(pdag2, 0);
closure2 = transitiveClosurePag_mex(pdag2);

sat2 = ~allKnown(pdag2) && ~hasCycle(closure2) && ~PKViolated(pdag2,PK,closure2) && search_all(pdag2, PK);

sat = sat1 || sat2;

end

function [sat pdag] = hardrule(pdag, PK)
[X,Y] = find(pdag == 1,1);

if(isempty(X))
    closure = transitiveClosurePag_mex(pdag);
    sat = ~hasCycle(closure) && ~PKViolated(pdag,PK,closure);
    return;
end

% Test arrow
pdag1 = pdag;
pdag1(X,Y) = 2;
pdag1(Y,X) = 3;
pdag1 = PC_rules_apply(pdag1, 0);
closure1 = transitiveClosurePag_mex(pdag1);

if(~allKnown(pdag1) && ~hasCycle(closure1) && ~PKViolated(pdag1,PK,closure1))
    [sat pdag1] = hardrule(pdag1,PK);
    if(sat)
        pdag = pdag1;
        return;
    end
end

% Test tail
pdag2 = pdag;
pdag2(X,Y) = 3;
pdag2(Y,X) = 2;
pdag2 = PC_rules_apply(pdag2, 0);
closure2 = transitiveClosurePag_mex(pdag2);

if(~allKnown(pdag2) && ~hasCycle(closure2) && ~PKViolated(pdag2,PK,closure2))
    [sat pdag2] = hardrule(pdag2,PK);
    if(sat)
        pdag = pdag2;
        return;
    end
end

sat = false;

end
