function [sepSets] = findTrivialSepSets_Mag(Mag)
% Input
%   Mag: NxN adjacency matrix
%       if Mag(i,j) == 2: endpoint from i to j is an arrowhead
%       if Mag(i,j) == 3: endpoint from i to j is a tail
%       if Mag(i,j) == 0: i and j are not adjacent
%
% Output
%   sepSets: NxNxN boolean matrix
%       if sepSets(i,j,k) == 1: vertex k is in the separating set of
%                               vertices i and j

dir = Mag == 2 & Mag' == 3;
bidir = Mag == 2 & Mag' == 2;

nnodes = length(Mag);
sepSets = false(nnodes,nnodes,nnodes);
closure = transitiveClosureSparse_mex(sparse(dir));

for i = 1:nnodes
    for j = i+1:nnodes
        if(~Mag(i,j))
            if(closure(i,j))
                sepSet = dir(:,j)';
            elseif(closure(j,i))
                sepSet = dir(:,i)';
            else
                sepSet = dir(:,i)' | dir(:,j)' | FindBidirectSepSet(dir,bidir,i,j,closure,false(1,nnodes),false(1,nnodes)) | FindBidirectSepSet(dir,bidir,j,i,closure,false(1,nnodes),false(1,nnodes));
            end
            sepSets(i,j,:) = sepSet;
            sepSets(j,i,:) = sepSet;
        end
    end
end
end

function [sepSet visited] = FindBidirectSepSet(dir,bidir,i,j,closure,sepSet,visited)
    visited(i) = true;
    % A vertex v is added to the m-separating set if:
    % a) it is connected by a bi-directed edge to vertex i and
    % b) is an ancestor of vertex j
    % Additionally all the parents of v are added to the separating set.
    % The same procedure is repeated for v recursively.
    
    V = find(bidir(:,i)' & ~visited & closure(:,j)');
    sepSet(V) = true;    
    visited(V) = true;
    
    for v = V
        [sepSet visited] = FindBidirectSepSet(dir,bidir,v,j,closure,sepSet,visited);
        sepSet = sepSet | dir(:,v)';
    end
end
