function bool = independenciesViolated(sepSet, Mag, closure)
% Checks whether all M-Separations of sepSet hold in Mag

[X Y] = find(Mag == 0);
n = length(X);
bool = false;
for i = 1:n
    if(X(i) ~= Y(i))
        if(~isMSeparated(Mag,X(i),Y(i),squeeze(sepSet(X(i),Y(i),:))',closure))
            bool = true;
            return;
        end
    end
end
end

function bool = isMSeparated(Mag, X, Y, Z, closure)

nnodes = length(Mag);
visited = false(nnodes,nnodes);

Q = zeros(nnodes*nnodes,2);

if(Mag(X,Y))
    bool = false;
    return;
end

U = find(Mag(X,:));
len = length(U);
Q(1:len,1) = X;
Q(1:len,2) = U;
curQ = len;
visited(X,:) = true;

des = false(1,nnodes);
for i = 1:nnodes
    des(i) = Z(i) || any(closure(i,:) & Z);
end

while(curQ)
    v = Q(curQ,1);
    u = Q(curQ,2);
    curQ = curQ-1;

    U = [];
    for i = 1:nnodes
        if(v ~= i && Mag(u,i) && ~visited(u,i))
            cond = Mag(v,u) == 2 && Mag(i,u) == 2;
            if((~cond && ~Z(u)) || (cond && des(u)))
                if(i == Y)
                    bool = false;
                    return;
                end
                U = [U i];
            end
        end
    end
    len = length(U);
    if(len > 0)
        Q(curQ+1:curQ+len,1) = u;
        Q(curQ+1:curQ+len,2) = U;
        curQ = curQ+len;
        visited(u,U) = true;
    end
end

bool = true;
end
